@@ -6,7 +6,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2017 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2018 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ static unsigned toeplitz_det(unsigned size)
 
 	// dirty consistency check of result:
 	if (!tdet.subs(a==0).subs(b==0).is_zero()) {
-		clog << "Determaint of Toeplitz matrix " << endl
+		clog << "Determinant of Toeplitz matrix " << endl
 		     << "M==" << M << endl
 		     << "was miscalculated: det(M)==" << tdet << endl;
 		++result;
@@ -76,15 +76,10 @@ unsigned time_toeplitz()
 
 	cout << "timing determinant of polyvariate symbolic Toeplitz matrices" << flush;
 
-	vector<unsigned> sizes;
+	vector<unsigned> sizes = {7, 8, 9, 10};
 	vector<double> times;
 	timer longines;
 
-	sizes.push_back(7);
-	sizes.push_back(8);
-	sizes.push_back(9);
-	sizes.push_back(10);
-
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i) {
 		int count = 1;
 		longines.start();
