@@ -7,7 +7,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2017 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2018 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ static unsigned vandermonde_det(unsigned size)
 			vanddet *= M(i,1) - M(j,1);
 
 	if (expand(det - vanddet) != 0) {
-		clog << "Determaint of Vandermonde matrix " << endl
+		clog << "Determinant of Vandermonde matrix " << endl
 		     << "M==" << M << endl
 		     << "was miscalculated: det(M)==" << det << endl;
 		++result;
@@ -73,15 +73,10 @@ unsigned time_vandermonde()
 	
 	cout << "timing determinant of univariate symbolic Vandermonde matrices" << flush;
 	
-	vector<unsigned> sizes;
+	vector<unsigned> sizes = {8, 10, 12, 14};
 	vector<double> times;
 	timer swatch;
 	
-	sizes.push_back(8);
-	sizes.push_back(10);
-	sizes.push_back(12);
-	sizes.push_back(14);
-	
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i) {
 		int count = 1;
 		swatch.start();
