@@ -3,7 +3,7 @@
  *  Implementation of abstract derivatives of functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2017 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2018 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(fderivative, function,
   print_func<print_context>(&fderivative::do_print).
+  print_func<print_latex>(&fderivative::do_print_latex).
   print_func<print_csrc>(&fderivative::do_print_csrc).
   print_func<print_tree>(&fderivative::do_print_tree))
 
@@ -111,6 +112,23 @@ void fderivative::do_print(const print_context & c, unsigned level) const
 	printseq(c, '(', ',', ')', exprseq::precedence(), function::precedence());
 }
 
+void fderivative::do_print_latex(const print_context & c, unsigned level) const
+{
+	int order=1;
+	c.s << "\\partial_{";
+	auto i = parameter_set.begin(), end = parameter_set.end();
+	--end;
+	while (i != end) {
+		++order;
+		c.s << *i++ << ",";
+	}
+	c.s << *i << "}";
+	if (order>1)
+		c.s << "^{" << order << "}";
+	c.s << "(" << registered_functions()[serial].TeX_name << ")";
+	printseq(c, '(', ',', ')', exprseq::precedence(), function::precedence());
+}
+
 void fderivative::do_print_csrc(const print_csrc & c, unsigned level) const
 {
 	c.s << "D_";
